package statistics

import (
	"errors"
	"strconv"
	"strings"
	"time"
)

// CardCouponStatistics 商品统计信息
type CardCouponStatistics struct {
	CaseName   string
	CName      string
	TypeName   string
	SendType   string
	Price      string
	SentCount  int
	UsedCount  int
	CreateDate time.Time
}

// CardCouponStatistics 获取商品统计信息
func (m *StatisticsDAO) CardCouponStatistics(caseids, ctype, name string, page, pageSize int) ([]CardCouponStatistics, error) {
	var cardCoupons []CardCouponStatistics
	couponsql := `SELECT
		b.case_name,
		b.case_id,
		a.coupon_name AS c_name,
		(
			CASE
			WHEN coupon_type = 'course' THEN
				'课程优惠券'
			ELSE
				'商品优惠券'
			END
		) type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	cardsql := `SELECT
		b.case_name,
		b.case_id,
		a.card_name AS c_name,
		'课程体验卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	gymcardsql := `SELECT
		b.case_name,
		b.case_id,
		a.gym_card_name,
		'健身卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_gym_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := ""
	if ctype != "" {
		switch ctype {
		case "card":
			sql = "select * from (" + cardsql + ") tab"
			break
		case "goodscoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='goods') tab"
			break
		case "coursecoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='course') tab"
			break
		case "gym":
			sql = "select * from (" + gymcardsql + ") tab"
			break
		default:
			return nil, errors.New("类型不正确")
		}
	} else {
		sql = "select * from (" + cardsql + " union " + couponsql + " union " + gymcardsql + ") tab"
	}

	if name != "" {
		sql = sql + ` and c_name like '%` + name + `%'`
	}
	sql = sql + ` order by create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponStatisticsCount 获取总数
func (m *StatisticsDAO) CardCouponStatisticsCount(caseids, ctype, name string) (int, error) {
	var cardCoupons []CardCouponStatistics
	couponsql := `SELECT
		b.case_name,
		b.case_id,
		a.coupon_name AS c_name,
		(
			CASE
			WHEN coupon_type = 'course' THEN
				'课程优惠券'
			ELSE
				'商品优惠券'
			END
		) type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	cardsql := `SELECT
		b.case_name,
		b.case_id,
		a.card_name AS c_name,
		'课程体验卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	gymcardsql := `SELECT
		b.case_name,
		b.case_id,
		a.gym_card_name,
		'健身卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_gym_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := ""
	if ctype != "" {
		switch ctype {
		case "card":
			sql = "select * from (" + cardsql + ") tab"
			break
		case "goodscoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='goods') tab"
			break
		case "coursecoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='course') tab"
			break
		case "gym":
			sql = "select * from (" + gymcardsql + ") tab"
			break
		default:
			return 0, errors.New("类型不正确")
		}
	} else {
		sql = "select * from (" + cardsql + " union " + couponsql + " union " + gymcardsql + ") tab"
	}

	if name != "" {
		sql = sql + ` and c_name like '%` + name + `%'`
	}
	err := m.db.Sql(sql).Find(&cardCoupons)
	return len(cardCoupons), err
}
